# Rank Tree
#
# * Construct a ranktree r from an array of n elements
# * find returns the element at index (rank) k
# * remove removes the element at rank k
# * len(r) returns the current size

class _Node():
  def __init__(self, element, size=1, left=None, right=None):
    self.element = element
    self.size = size
    self.left = left
    self.right = right

  def __str__(self):
    ls = str(self.left) if self.left else ""
    rs = str(self.right) if self.right else ""
    return ls + str(self.element) + ("(%d)" % self.size) + " " + rs
    #return ls + str(self.element) + " " + rs

  def recompute_size(self):
    ls = self.left.size if self.left else 0
    rs = self.right.size if self.right else 0
    self.size = ls + 1 + rs

  # Find node at rank k in the subtree rooted at this node
  def _find(self, k):
    ls = self.left.size if self.left else 0
    if k < ls:
      return self.left._find(k)
    elif k == ls:
      return self
    else:
      return self.right._find(k - ls - 1)

  # Find the node at rank 0 in the subtree rooted at this node
  def _find_first(self):
    p = self
    while p.left is not None:
      p = p.left
    return p

  # Remove node at rank 0 in the subtree rooted at this node
  # Returns the new root.
  def _remove_first(self):
    if self.left is None:
      return self.right
    else:
      self.left = self.left._remove_first()
      self.recompute_size()
      return self

  # Remove node at rank k in the subtree rooted at this node
  # Returns the new root.
  def _remove(self, k):
    ls = 0 if self.left is None else self.left.size
    if k < ls:
      self.left = self.left._remove(k)
    elif k > ls:
      self.right = self.right._remove(k - ls - 1)
    elif self.left is not None and self.right is not None:
      # Need to remove self, but has two children
      self.element = self.right._find_first().element
      self.right = self.right._remove_first()
    else:
      # Need to remove self, which has zero or one child
      return self.left if self.left else self.right
    self.recompute_size()
    return self

class RankTree():
  def __init__(self, a):
    self._root = self._build_tree(a, 0, len(a) - 1)

  def __len__(self): 
    return self._root.size if self._root else 0

  # Find item at rank k in the tree.
  # throws exception if k is out of range
  def find(self, k):
    if k < 0 or k >= len(self):
      raise ValueError(k)
    return self._root._find(k).element
  
  # Remove element at rank k from the tree.
  def remove(self, k):
    if k < 0 or k >= len(self):
      raise ValueError(k)
    self._root = self._root._remove(k)

  def __str__(self):
    return "[ " + (str(self._root) if self._root else "") + "]"

  def _build_tree(self, a, lo, hi):
    if hi < lo:
      return None
    elif lo == hi:
      return _Node(a[lo])
    else:
      mid = (lo + hi) // 2
      lhs = self._build_tree(a, lo, mid - 1)
      rhs = self._build_tree(a, mid + 1, hi)
      t = _Node(a[mid], 1, lhs, rhs)
      t.recompute_size()
      return t

